// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//	DOOM graphics stuff for X11, UNIX.
//
//-----------------------------------------------------------------------------

static const char
rcsid[] = "$Id: i_x.c,v 1.6 1997/02/03 22:45:10 b1 Exp $";

#include <stdlib.h>
#include <unistd.h>
//#include <sys/ipc.h>
//#include <sys/shm.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include <stdarg.h>
#include <sys/time.h>
#include <sys/types.h>
//#include <sys/socket.h>

//#include <netinet/in.h>
#include <errno.h>
#include <signal.h>

#include "../stage2boot/gfxlib.h"

#include "doomstat.h"
#include "i_system.h"
#include "v_video.h"
#include "m_argv.h"
#include "d_main.h"

#include "sha.h"

#include "doomdef.h"

#include "sha_check.h"

#define POINTER_WARP_COUNTDOWN	1



#define NUM_BUFFERS     32

static char * buffer[ NUM_BUFFERS ];
static int current_buffer = 0 ;


void I_ShutdownGraphics(void)
{
  // Detach from X server
  /*if (!XShmDetach(X_display, &X_shminfo))
	    I_Error("XShmDetach() failed in I_ShutdownGraphics()");

  // Release shared memory.
  shmdt(X_shminfo.shmaddr);
  shmctl(X_shminfo.shmid, IPC_RMID, 0);*/

  // Paranoia.
}



//
// I_StartFrame
//
void I_StartFrame (void)
{
    // er?

}

static int	lastmousex = 0;
static int	lastmousey = 0;
boolean		mousemoved = false;
boolean		shmFinished;

//
// I_StartTic
//
void I_StartTic (void)
{
    /* Poll for events (calling I_GetEvent while events are in
     * the queue) and stop the mouse pointer leaving the window
     * as a dirty keep-focus hack
     */
    I_GetEvent();
}


//
// I_UpdateNoBlit
//
void I_UpdateNoBlit (void)
{
    // what is this?
}

int  I_GetTime (void)
{
    static unsigned time = 0;
    static unsigned t0 = 0;
    unsigned t1 = vga[ 0 ];

    if ( t1 < t0 ) {
        time += 0x100;
    }
    t0 = t1 ;
    return (int) ( time | t1 ) ;
}

//
// I_FinishUpdate
//
void I_FinishUpdate (void)
{

    //volatile char * note = (char *) 0x83e00007;
    static int	update = 0;
    static int	lasttic;
    int		tics;
    int		i;
    // UNUSED static unsigned char *bigscreen=0;

    // draws little dots on the bottom of the screen
    if (1)
    {
	i = I_GetTime();
	tics = i - lasttic;
	lasttic = i;
	if (tics > 20) tics = 20;

	for (i=0 ; i<tics*2 ; i+=2)
	    screens[0][ (SCREENHEIGHT-1)*SCREENWIDTH + i] = 0xff;
	for ( ; i<20*2 ; i+=2)
	    screens[0][ (SCREENHEIGHT-1)*SCREENWIDTH + i] = 0x0;
    
    }

    //*note = 1;


    /* Here is where screens[0] is blitted onto the X display */
    //if (( update % 64 ) == 63 )
#if 0
    {
        static unsigned count = 0;
        static unsigned failed = 0;
        SHA_INFO sha_info;

        sha_init(&sha_info);
        sha_update(&sha_info, screens[ 0 ], SCREENHEIGHT * SCREENWIDTH);
        sha_final(&sha_info);
        count ++ ;
	    printf ( "check %04x: %08x: ",
                count, 
                sha_info.digest[ 0 ]);
        if ( sha_check[ count ] != sha_info.digest[ 0 ] )
        {
            printf ( "failed\n" ) ;
            failed ++ ;
        } else {
            printf ( "verified\n" ) ;
        }
        if (( count > 1024 ) || ( failed > 10 ))
        {
            if ( failed != 0 )
            {
                printf ( "test failed\n" ) ;
                exit ( 1 ) ;
            } else {
                printf ( "test passed\n" ) ;
                exit ( 0 ) ;
            }
        }
    }
#endif
/*
    {
        static unsigned count = 0;
        char name [ 32 ] ;
        int fd;

        sprintf ( name, "bins/%04x.bin", count );
        count ++ ;
        fd = open ( name , O_CREAT | O_WRONLY | O_TRUNC , 0644  ) ;
        if ( fd >= 0 )
        {
            write ( fd , screens[ 0 ], SCREENHEIGHT * SCREENWIDTH ) ;
            write ( fd , cur_palette, 768 ) ;
            close ( fd ) ;
        }
    }
*/
    //update ++;
    {
        current_buffer = ( current_buffer + 1 ) % NUM_BUFFERS ;
        memcpy(buffer[ current_buffer ], 
                screens[ 0 ], SCREENHEIGHT * SCREENWIDTH);
        vga[ 0 ] = buffer[ current_buffer ];
    }
}


//
// I_ReadScreen
//
void I_ReadScreen (byte* scr)
{
    memcpy (scr, screens[0], SCREENWIDTH*SCREENHEIGHT);
}


//
// Palette stuff.
//

//
// I_SetPalette
//
void I_SetPalette (byte* palette)
{
    unsigned c, r, g, b;

    for (c = 0; c < 256; c++) {
        r = (unsigned char) palette[ (c * 3) + 0 ] ;
        g = (unsigned char) palette[ (c * 3) + 1 ] ;
        b = (unsigned char) palette[ (c * 3) + 2 ] ;
        vga [ 1 ] = ((r >> 3) & 0x1f)
                | (((g >> 3) & 0x1f) << 5)
                | (((b >> 3) & 0x1f) << 10)
                | (c << 16) | (1 << 31);
    }
    // palette is an array of 256 RGB triples.
    // i.e. 768 bytes
}


void I_InitGraphics(void)
{

    char*		d;
    int			n, i;
    int			pnum;
    int			x=0;
    int			y=0;
    
    // warning: char format, different type arg
    char		xsign=' ';
    char		ysign=' ';
    
    int			oktodraw;
    unsigned long	attribmask;
    int			valuemask;
    static int		firsttime=1;

    if (!firsttime)
	return;
    firsttime = 0;

#if 0
    signal(SIGINT, (void (*)(int)) I_Quit);
#endif

    for (i = 0; i < NUM_BUFFERS; i ++) {
        buffer[ i ] = malloc(SCREENWIDTH * SCREENHEIGHT);
        memset(buffer[ i ], 0, SCREENWIDTH * SCREENHEIGHT);
    }
}


